@@ -2,21 +2,21 @@ import urllib.parse
 import requests
 import zalando_kubectl.utils
 
-from zalando_kubectl.utils import auth_headers
+from zalando_kubectl.utils import auth_headers, Environment
 
 
-def emergency_service_url(config):
+def emergency_service_url(env: Environment) -> str:
     # emergency-access service URL isn't stored anywhere in the cluster metadata,
     # so we need to build it manually by taking the API server URL and replace the first
     # component with emergency-access.
-    api_server_host = urllib.parse.urlparse(zalando_kubectl.utils.get_api_server_url(config)).hostname
+    api_server_host = urllib.parse.urlparse(zalando_kubectl.utils.get_api_server_url(env)).hostname
     _, cluster_domain = api_server_host.split(".", 1)
     return "https://emergency-access-service.{}".format(cluster_domain)
 
 
-def create(config, access_type, reference_url, user, reason):
+def create(env: Environment, access_type, reference_url, user, reason):
     response = requests.post(
-        "{}/access-requests".format(emergency_service_url(config)),
+        "{}/access-requests".format(emergency_service_url(env)),
         json={"access_type": access_type, "reference_url": reference_url, "user": user, "reason": reason},
         headers=auth_headers(),
         timeout=20,
@@ -24,8 +24,8 @@ def create(config, access_type, reference_url, user, reason):
     _handle_http_error(response)
 
 
-def approve(config, username):
-    service_url = emergency_service_url(config)
+def approve(env: Environment, username):
+    service_url = emergency_service_url(env)
     response = requests.post(
         "{}/access-requests/{}".format(service_url, urllib.parse.quote_plus(username)),
         headers=auth_headers(),
@@ -35,23 +35,32 @@ def approve(config, username):
     return response.json()["reason"]
 
 
+def _extract_error(response):
+    error_obj = response.json()
+    title = error_obj.get("title")
+    detail = error_obj.get("detail")
+    message = "\n".join(filter(None, [title, detail]))
+    if response.status_code != 401:
+        message = "[{code}] {message}".format(code=response.status_code, message=message)
+    return message
+
+
 def _handle_http_error(response):
     try:
         response.raise_for_status()
-    except requests.exceptions.HTTPError:
-        # Try to see if we can extract a nice error message
-        error_obj = response.json()
-        title = error_obj.get("title")
-        detail = error_obj.get("detail")
-        message = "\n".join(filter(None, [title, detail]))
-        raise Exception("[{code}] {message}".format(code=response.status_code, message=message))
+    except requests.exceptions.HTTPError as e:
+        # Try to see if we can extract a nice error message, or just raise the original error if we can't
+        try:
+            message = _extract_error(response)
+        except Exception:
+            raise e
+        else:
+            raise Exception(message)
 
 
-def get_all(config):
+def get_all(env: Environment):
     """Get all current access requests from the Emergency Access Service"""
-    response = requests.get(
-        "{}/access-requests".format(emergency_service_url(config)), headers=auth_headers(), timeout=20
-    )
+    response = requests.get("{}/access-requests".format(emergency_service_url(env)), headers=auth_headers(), timeout=20)
     _handle_http_error(response)
     access_requests = response.json()
     return combine_request_approved(access_requests["items"])
