@@ -7,6 +7,7 @@ import subprocess
 import sys
 import tarfile
 from pathlib import Path
+from zipfile import ZipFile
 
 import click
 import jwt
@@ -15,8 +16,19 @@ import stups_cli
 import stups_cli.config
 import zign.api
 from clickclick import Action, AliasedGroup
+from pkg_resources import resource_filename
+from subprocess import check_output
 
-from . import APP_NAME, APP_VERSION, KUBECTL_VERSION, KUBECTL_SHA512, STERN_VERSION, STERN_SHA256
+from . import (
+    APP_NAME,
+    APP_VERSION,
+    KUBECTL_VERSION,
+    KUBECTL_SHA512,
+    KUBELOGIN_VERSION,
+    KUBELOGIN_SHA256,
+    STERN_VERSION,
+    STERN_SHA256,
+)
 
 
 def auth_token():
@@ -49,6 +61,15 @@ def extract_tgz(filepath):
     return extract
 
 
+def extract_zip(filepath):
+    def extract(archive):
+        with ZipFile(io.BytesIO(archive), "r") as zf:
+            with zf.open(zf.infolist()[0].filename) as file:
+                return file.read()
+
+    return extract
+
+
 class ExternalBinary:
     def __init__(self, env, name, url_template, version, checksum_algo, checksum, extract, exec_name=None):
         self.env = env
@@ -70,6 +91,18 @@ class ExternalBinary:
     def download(self):
         binary = self._binary()
 
+        platform = sys.platform  # linux or darwin
+        if self.name == "kubectl" and platform == "darwin":
+            if binary.exists():
+                kubectl_version = check_output([str(binary), "version", "--client", "--short"], cwd=Path.cwd())
+                return str(binary) if "1.21.3" in kubectl_version.decode("UTF-8") else os.remove(binary)
+
+            filename = resource_filename(__name__, "binaries/kubectl")
+            with open(filename, "rb") as f:
+                data = f.read()
+                self.create_binary(binary, data)
+            return str(binary)
+
         if not binary.exists():
             try:
                 binary.parent.mkdir(parents=True)
@@ -78,7 +111,6 @@ class ExternalBinary:
                 # "exist_ok" was introduced with 3.5
                 pass
 
-            platform = sys.platform  # linux or darwin
             arch = "amd64"  # FIXME: hardcoded value
             url = self.url_template.format(version=self.version, os=platform, arch=arch)
             with Action("Downloading {} {}...".format(self.name, self.version)) as act:
@@ -91,17 +123,20 @@ class ExternalBinary:
                 if m.hexdigest() != self.checksum[platform]:
                     act.fatal_error("CHECKSUM MISMATCH")
 
-                # add random suffix so we can do an atomic write and rename
-                random_suffix = "".join(random.choice(string.ascii_letters + string.digits) for _ in range(8))
-                local_file = binary.with_name("{}.download-{}".format(binary.name, random_suffix))
-                with local_file.open("wb") as fd:
-                    fd.write(self.extract(data))
-                    fd.flush()
-                local_file.chmod(0o755)
-                local_file.rename(binary)
+                self.create_binary(binary, self.extract(data))
 
         return str(binary)
 
+    def create_binary(self, binary, data):
+        # add random suffix so we can do an atomic write and rename
+        random_suffix = "".join(random.choice(string.ascii_letters + string.digits) for _ in range(8))
+        local_file = binary.with_name("{}.download-{}".format(binary.name, random_suffix))
+        with local_file.open("wb") as fd:
+            fd.write(data)
+            fd.flush()
+        local_file.chmod(0o755)
+        local_file.rename(binary)
+
     def run(self, args, input=None, stdout=None, stderr=None, check=False):
         if self.exec_name is not None:
             exec_name = self.exec_name
@@ -116,7 +151,12 @@ class ExternalBinary:
         cmdline.extend(args)
 
         return subprocess.run(
-            cmdline, input=input, stdout=stdout, stderr=stderr, check=check, executable=self.download()
+            cmdline,
+            input=input,
+            stdout=stdout,
+            stderr=stderr,
+            check=check,
+            executable=self.download(),
         )
 
     def exec(self, *args):
@@ -156,6 +196,21 @@ class Environment:
             ),
         )
 
+        kubelogin_template = "https://github.com/int128/kubelogin/releases/download/{version}/kubelogin_{os}_{arch}.zip"
+        self.kubelogin = ExternalBinary(
+            self,
+            "kubelogin",
+            kubelogin_template,
+            KUBELOGIN_VERSION,
+            hashlib.sha256,
+            KUBELOGIN_SHA256,
+            extract_zip(
+                "kubelogin_{version}_{os}_{arch}/kubelogin".format(
+                    version=KUBELOGIN_VERSION, os=sys.platform, arch="amd64"
+                )
+            ),
+        )
+
     def store_config(self):
         stups_cli.config.store_config(self.config, APP_NAME)
 
